/*
 * Copyright (C) 2004 Patrik Karlsson <patrik@cqure.net>
 * 
 * This file is part of the oscanner framework
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 */

package cqure.repeng;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.ImageIcon;

import java.net.URL;
import java.io.*;
import java.awt.Dimension;
import java.awt.GridLayout;

public class ReportViewer extends JPanel implements TreeSelectionListener {
	private JEditorPane oHtmlPane;

	private JTree tree;

	private URL helpURL;

	private static boolean DEBUG = false;

	private ReportTree oTree = null;

	private static String m_sFile = null;

	public static ReportTree loadFromFile(String sName) {

		FileInputStream oFIS = null;
		ObjectInputStream oOIS = null;
		ReportTree oTree = null;

		try {
			oFIS = new FileInputStream(sName);
			oOIS = new ObjectInputStream(oFIS);

			oTree = (ReportTree) oOIS.readObject();
			oFIS.close();
			oOIS.close();
		} catch (Exception e) {
			System.err.println("[x] Failed to load report");
			e.printStackTrace();
		}

		return oTree;

	}

	public ReportViewer(String sFile) {
		super(new GridLayout(1, 0));

		ReportTree oTree = new ReportTree();
		oTree = loadFromFile(sFile);

		tree = new JTree(oTree);
		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);

		tree.addTreeSelectionListener(this);
		tree.setCellRenderer(new ReportTreeCellRenderer());

		JScrollPane treeView = new JScrollPane(tree);

		oHtmlPane = new JEditorPane();
		oHtmlPane.setEditable(false);
		JScrollPane htmlView = new JScrollPane(oHtmlPane);

		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setTopComponent(treeView);
		splitPane.setBottomComponent(htmlView);

		Dimension minimumSize = new Dimension(100, 50);
		htmlView.setMinimumSize(minimumSize);
		treeView.setMinimumSize(minimumSize);
		splitPane.setDividerLocation(300);
		splitPane.setPreferredSize(new Dimension(500, 500));

		add(splitPane);
	}

	public void valueChanged(TreeSelectionEvent e) {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree
				.getLastSelectedPathComponent();

		String sTmp = "";
		ReportObject oRo = null;

		if (node == null)
			return;

		Object nodeInfo = node;

		if (nodeInfo instanceof ReportObject)
			oRo = (ReportObject) nodeInfo;

		if (oRo != null) {

			if (oRo.getValue() != null)
				sTmp = (String) oRo.getValue();
			else
				sTmp = oRo.getPropertiesAsString();

			oHtmlPane.setText(sTmp);

		}

	}

	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = ReportViewer.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	public static void createAndShowGUI(String sFile) {
		JFrame.setDefaultLookAndFeelDecorated(true);
		JFrame frame = new JFrame("ReportViewer");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		ReportViewer newContentPane = null;

		if (sFile != null)
			newContentPane = new ReportViewer(sFile);
		else
			newContentPane = new ReportViewer(m_sFile);

		newContentPane.setOpaque(true); //content panes must be opaque
		frame.setContentPane(newContentPane);

		frame.pack();
		frame.setVisible(true);
	}

	public static void main(String[] args) {

		if (args.length < 1) {
			System.err.println("Reportviewer v.0.0.1 by patrik@cqure.net");
			System.err.println("----------------------------------------");
			System.err.println("Reportviewer [filename]");
			System.exit(1);
		}

		m_sFile = args[0];

		javax.swing.SwingUtilities.invokeLater(new Runnable() {

			public void run() {
				createAndShowGUI(null);
			}
		});

	}

}